@@ -1,5 +1,6 @@
 #
 # Copyright 2014 Andrew Ayer
+# Copyright 2016-2020 Chris Lamb <lamby@debian.org>
 #
 # This file is part of strip-nondeterminism.
 #
@@ -21,10 +22,19 @@ package File::StripNondeterminism::handlers::zip;
 use strict;
 use warnings;
 
+use File::Basename qw(basename);
 use File::Temp;
 use File::StripNondeterminism;
 use Archive::Zip qw/:CONSTANTS :ERROR_CODES/;
-use Sub::Override;
+
+=head1 DEPRECATION PLANS
+
+Given that so many libraries and utilities generate ZIP archives themselves
+(rather than delegating to one or two central libraries) it is infeasible to
+get this upstreamed everywhere. There are also countless formats that generate
+ZIP files under different names (notably Jar archives and its many variants).
+
+=cut
 
 # A magic number from Archive::Zip for the earliest timestamp that
 # can be represented by a Zip file.  From the Archive::Zip source:
@@ -55,13 +65,19 @@ sub normalize_member($$) {
 
 	# Extract the member to a temporary file.
 	my $tempdir = File::Temp->newdir();
-	my $filename = "$tempdir/member";
+	my $filename = "$tempdir/" . basename($member->fileName());
 	my $original_size = $member->compressedSize();
 	$member->extractToFileNamed($filename);
 	chmod(0600, $filename);
 	$member->{'compressedSize'} = $original_size
 	  ; # Work around https://github.com/redhotpenguin/perl-Archive-Zip/issues/11
 
+	$normalizer = File::StripNondeterminism::get_normalizer_for_file($filename) unless defined $normalizer;
+	if (not defined $normalizer) {
+		warn "strip-nondeterminism: unknown file type of " . $member->fileName();
+		return;
+	}
+
 	# Normalize the temporary file.
 	if ($normalizer->($filename)) {
 		# Normalizer modified the temporary file.
@@ -200,6 +216,16 @@ sub normalize {
 	$canonical_time = SAFE_EPOCH
 	  if not defined $canonical_time or $canonical_time < SAFE_EPOCH;
 	my @filenames = sort $filename_cmp $zip->memberNames();
+	if (exists($options{filename_filter})) {
+		@filenames = $options{filename_filter}->(@filenames);
+		# Remove any members deleted by the custom filter
+		my %seen = map { $_ => 1 } @filenames;
+		for my $filename ($zip->memberNames()) {
+			next if $seen{$filename};
+			print STDERR "Removing $filename from $zip_filename\n";
+			$zip->removeMember($filename);
+		}
+	}
 	for my $filename (@filenames) {
 		my $member = $zip->removeMember($filename);
 		if ($member->isEncrypted()) {
@@ -238,17 +264,20 @@ sub normalize {
 	my @overrides = map {
 		my $full_name = "Archive::Zip::Member::$_";
 		my $orig_sub = \&$full_name;
-		Sub::Override->new(
-			$full_name => sub {
-				my $result = $orig_sub->(@_);
-				return defined($result) ?
-					normalize_extra_fields($canonical_time, $result) : $result;
-			}
-		);
+
+		no warnings qw(redefine);
+		no strict qw(refs);
+		*{"Archive\::Zip\::Member\::$_"} = sub {
+			my $result = $orig_sub->(@_);
+			return defined($result) ?
+				normalize_extra_fields($canonical_time, $result) : $result;
+		};
+
+		sub { *{"Archive\::Zip\::Member\::$_"} = $orig_sub };
 	} qw(cdExtraField localExtraField);
 
 	return 0 unless $zip->overwrite() == AZ_OK;
-	$_->restore for @overrides;
+	$_->() for @overrides;
 	chmod($old_perms, $zip_filename);
 	return 1;
 }
